<?php
/**
 * @file
 * Admin functions for the Subscribe module.
 *
 */

function subscribe_overview() {
  $header = array(array('data' => t('Name')), array('data' => t('Endpoint')), array('data' => t('Last received')),  array('data' => t('Nodes received')), array('data' => 'Operations', 'colspan' => '3'));  
  $result = db_query('SELECT * FROM {subscribe_subscriptions}');
  $rows = array();
  while ($sub = db_fetch_object($result)) {
    $count = db_result(db_query('SELECT COUNT(nid) FROM {subscribe_node} WHERE sid = %d', $sub->sid));
    $rows[] = array(
      array('data' => $sub->name),
      array('data' => l($sub->url, $sub->url, array('absolute' => TRUE))),
      array('data' => $sub->last_update ? format_date($sub->last_update) : t('Never')),
      array('data' => $count),
      array('data' => l(t('Edit'), 'admin/content/subscribe/edit/'. $sub->sid)),
      array('data' => l(t('Pull'), 'admin/content/subscribe/pull/'. $sub->sid)),
      array('data' => l(t('Delete'), 'admin/content/subscribe/delete/'. $sub->sid)),
    );
  }
  $output = count($rows) ? theme('table', $header, $rows) : t('You currently have no subscriptions.') . '</p><p>' . t('Would you like to') . ' ' . l(t('subscribe to a channel'), 'admin/content/subscribe/add') . '?</p>';

  return $output;
}

function subscribe_admin_subscription_form_add($form_state) {
  if (empty($form_state['storage']['step'])) {
    // we are coming in without a step, so default to step 1
    $form_state['storage']['step'] = 1;
  }
  
  $form_values = $form_state['storage']['values'][$form_state['storage']['step']];
  
  switch ($form_state['storage']['step']) {
    case 1:
      $form['header'] = array(
        '#type' => 'markup',
        '#value' => '<h3>'. t('Step 1 - Publisher url') .'</h3>',
      );

      $form['url'] = array(
        '#type' => 'textfield',
        '#title' => t('URL of site including XML-RPC endpoint'),
        '#size' => '70',
        '#maxlength' => '255',
        '#description' => t('E.g. http://example.com/xmlrpc.php'),
        '#default_value' => isset($form_values['url']) ? $form_values['url'] : 'http://'
      );

      break;

    case 2:
      $form['header'] = array(
        '#type' => 'markup',
        '#value' => '<h3>'. t('Step 2 - Select a channel') .'</h3>',
      );

      $result = subscribe_xmlc_channel_list($form_state['storage']['values'][1]['url']);
      $form_state['storage']['channels'] = $result;
      $channels = array();
      $channels[-1] = t('Another channel');
      if (count($result)) {
        foreach ($result as $channel) {
          $channels[$channel['id']] = $channel['name'] . ' ('. ($channel['authtype'] == PUBLISH_AUTHTYPE_NONE ? t('Open channel') : t('Requires authentication')) .')';
        }
      }

      $form['channel'] = array(
        '#type' => 'radios',
        '#title' => t('Channels'),
        '#options' => $channels,
        '#description' => t('This is a list of the channels the publisher is advertising.'),
        '#default_value' => isset($values['channels']) ? $values['channels'] : -1,
        '#ahah' => array(
          'event' => 'change',
          'path' => 'admin/content/subscribe/js',
          'wrapper' => 'channel-info',
          'method' => 'replace',
          'effect' => 'fade',
        ),
      );
      
      $form['container_begin'] = array(
        '#type' => 'markup',
        '#value' => '<div id="channel-info">',
      );
      
      $form += _subscribe_admin_subscribtion_form_sub(-1, $result, $form_state['values']);

      $form['container_end'] = array(
        '#type' => 'markup',
        '#value' => '</div>',
      );

      break;
    
    case 3:
      $form['header'] = array(
        '#type' => 'markup',
        '#value' => '<h3>'. t('Step 3 - Filters and vocabularies') .'</h3>',
      );
      $channel = $form_state['storage']['msresult'][1];
      $vocabs = taxonomy_get_vocabularies();
      $vocabularies = array();
      
      foreach ($vocabs as $voc) {
        $vocabularies[$voc->vid] = $voc->name;
      }
      
      if (count($channel['filters'])) {
        $form['filters'] = array(
          '#type' => 'fieldset',
          '#title' => t('Filters'),
          '#description' => t('The channel supports the following filters. You can enter values for them to restrict the nodes returned.'),
        );
        
        foreach ($channel['filters'] as $name => $title) {
          $form['filters']['filter_'. $name] = array(
            '#type' => 'textfield',
            '#title' => $title,
          );
        }
      }

      if (count($channel['vocabularies'])) {
        $form['vocabularies'] = array(
          '#type' => 'fieldset',
          '#title' => t('Vocabularies'),
          '#description' => t('You can map remote vocabularies to local ones. Terms which do not exist locally will be created.'),
        );
        
        if (count($vocabularies)) {
          foreach ($channel['vocabularies'] as $vid => $title) {
            $form['vocabularies']['vocabulary_'. $vid] = array(
              '#type' => 'select',
              '#title' => t('Map !vocab to', array('!vocab' => $title)),
              '#options' => $vocabularies,
            );
          }
        }
        else {
          $form['vocabularies']['voc_info'] = array(
            '#type' => 'markup',
            '#value' => t('The publisher provides the following vocabularies: !vocabularies. To map them to local ones you need to have at least one vocabulary defined.', array('!vocabularies' => implode(', ', $channel['vocabularies']))),
          );
        }
        
        $form['pushpull'] = array(
          '#type' => 'radios',
          '#title' => t('Method'),
          '#default_value' => 0,
          '#options' => array(
            t('Push: Receive content from channel automatically when the channel decides to send it.'), 
            t('Pull: Receive content from channel only when you manually request it (your server must allow HTTP out).')
          ),
          '#description' => t('Choose how you would like this site to receive content.')
        );        
      }
      break;
  }
  
  if ($form_state['storage']['step'] > 1) {
    $form['previous'] = array(
      '#type' => 'submit',
      '#value' => t('Previous'),
    );
  }

  $form['next'] = array(
    '#type' => 'submit',
    '#value' => t('Next'),
  );  
  
  return $form;
}

function subscribe_admin_subscription_ahah($channel_id = NULL) {
  if ($channel_id == NULL) {
    $channel_id = $_POST['channel'];
  }
  
  $form_elements = array();
  $form_state = array('submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];

  if (!$form = form_get_cache($form_build_id, $form_state)) {
    exit();
  }

  $channels = $form_state['storage']['channels'];
  $form_elements = _subscribe_admin_subscribtion_form_sub($channel_id, $channels);
  foreach ($form_elements as $name => $element) {
    $form[$name] = $element;
  }
    
  form_set_cache($form_build_id, $form, $form_state);
  $form += array(
    '#post' => $_POST,
    '#programmed' => FALSE,
  );

  // Rebuild the form.
  $form = form_builder('subscribe_admin_subscription_form_add', $form, $form_state);
  $output = theme('status_messages');
  foreach ($form_elements as $name => $element) {
    $output .= drupal_render($form[$name]);
  }

  drupal_json(array('status' => TRUE, 'data' => $output));
}

function _subscribe_admin_subscribtion_form_sub($channel_id, $channels, $values = array()) {
  if ($channel_id != -1) {
    $form['channel_info'] = array(
      '#type' => 'markup',
      '#value' => theme('channel', $channels[$channel_id]),
    );
  }
  else {
    $form['channel_other'] = array(
      '#type' => 'markup',
      '#value' => t('To connect to a different channel than the ones in the list you need to enter the channel identifier and, optionally, a username and password if the channel requires them.'),
    );
    
    $form['channel_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Channel identifier'),
      '#default_value' => isset($values['channel_id']) ? $values['channel_id'] : '',
    );
  }
  
  if ($channel_id == -1 || $channels[$channel_id]['authtype']) {
    if ($channel_id != -1) {
      $form['channel_other'] = array(
        '#type' => 'markup',
        '#value' => t('This channel requires a username and password.'),
      );
    }

    $form['username'] = array(
      '#type' => 'textfield',
      '#title' => t('Username'),
      '#default_value' => isset($values['username']) ? $values['username'] : '',
    );

    $form['pass'] = array(
      '#type' => 'password',
      '#title' => t('Password'),
      '#default_value' => isset($values['pass']) ? $values['pass'] : '',
    );
  }
    
  return $form;
}

function subscribe_admin_subscription_form_add_validate($form, &$form_state) {
  switch ($form_state['storage']['step']) {
    case 1:
      if ($form_state['values']['url'] == '' || $form_state['values']['url'] == 'http://') {
        form_set_error('url', t('Please enter a url.'));
      }
      if (!valid_url($form_state['values']['url'])) {
        form_set_error('url', t('The url is not valid.'));
      }
      break;
      
    case 2:
      if ($form_state['values']['channel'] == -1) {
        return;
      }
      if ($form_state['storage']['channels'][$form_state['values']['channel']]['authtype'] && empty($form_state['values']['username'])) {
        form_set_error('username', t('Please enter a username.'));
        return;
      }

      if ($form_state['storage']['channels'][$form_state['values']['channel']]['authtype'] && empty($form_state['values']['pass'])) {
        form_set_error('pass', t('Please enter a password.'));
        return;
      }
      
      $result = subscribe_xmlc_may_subscribe($form_state['storage']['values'][1]['url'], $form_state['values']['channel'] == -1 ? $form_state['values']['channel_id'] : $form_state['values']['channel'], $form_state['values']['username'], $form_state['values']['pass']);
      if (!$result[0]) {
        form_set_error($form_state['values']['channel'] == -1 ? 'channel_id' : 'channel', t('You are not allowed to subscribe to this channel.'));
      }
      else {
        $form_state['storage']['msresult'] = $result;
      }
      break;
  }
}

function subscribe_admin_subscription_form_add_submit($form, &$form_state) {
  global $base_url;
  
  $form_state['storage']['values'][$form_state['storage']['step']] = $form_state['values'];

  if ($form_state['storage']['step'] == 3) {
    $data = array();
    $data['name']       = $form_state['storage']['msresult'][1]['name'];
    $data['base_url']   = $form_state['storage']['msresult'][2];
    $data['url']        = $form_state['storage']['values'][1]['url'];
    $data['username']   = $form_state['storage']['values'][2]['username'];
    $data['pass']       = $form_state['storage']['values'][2]['pass'];
    $data['channel_id'] = $form_state['storage']['values'][2]['channel'] == -1 ? $form_state['storage']['values'][2]['channel_id'] : $form_state['storage']['values'][2]['channel'];
    $data['method']     = $form_state['values']['pushpull'];
    $data['dest']       = $base_url .'/xmlrpc.php';
    $data['site_name']  = variable_get('site_name', 'Drupal');
    $data['site_url']  = $base_url;
        
    $data['filters'] = array();
    $data['vocabularies'] = array();
    foreach ($form_state['values'] as $name => $value) {
      if (strpos($name, 'filter_') === 0) {
        $data['filters'][str_replace('filter_', '', $name)] = $value;
      }
      if (strpos($name, 'vocabulary_') === 0) {
        $data['vocabularies'][str_replace('vocabulary_', '', $name)] = $value;
      }
    }
    
    $result = subscribe_xmlc_subscribe($data['url'], $data);
    if ($result['status'] = 'ok') {
      $data['rsid'] = $result['sid'];
      $data['skey'] = $result['key'];
      subscribe_subscription_save($data);
      drupal_set_message(t('Subscription to channel !name established.', array('!name' => theme('placeholder', $data['name']))));
      drupal_goto('admin/content/subscribe');
    }
    else {
      drupal_set_message(t('Could not subscribe.'), 'error');
      drupal_goto('admin/content/subscribe');
    }
  }
  else {
    if ($form_state['clicked_button']['#id']=='edit-previous') {
      $form_state['storage']['step']--;
    } 
    elseif ($form_state['clicked_button']['#id']=='edit-next') {
      $form_state['storage']['step']++;
    }
    $form_state['rebuild'] = TRUE;
  }
}

function subscribe_admin_subscription_form_delete($form_state, $sid) {
  $subscription = subscribe_subscription_load($sid);
  if (!$subscription) {
    drupal_set_message(t('Unable to load the specified subscription.'));
    drupal_goto('admin/content/subscribe');
  }
  
  return confirm_form(
    array(
      'subscription' => array(
        '#type' => 'value',
        '#value' => $subscription,
      ),
    ),
    t('Are you sure you want to remove the subscription to %subscription?', array('%subscription' => $subscription->base_url)),
    'admin/content/subscribe',
    t('This action cannot be undone.'),
    t('Delete subscription'),
    t('Cancel')
  );
}

function subscribe_admin_subscription_form_delete_submit($form, &$form_state) {
  $subscription = $form_state['values']['subscription'];
  
  subscribe_subscription_delete($subscription->sid);
  
  drupal_set_message(t('Deleted subscription to !subscription', array('!subscription' => $subscription->base_url)));
  watchdog('publish', 'Deleted subscription to !subscription', array('!subscription' => $subscription->base_url));
  
  $form_state['redirect'] = 'admin/content/subscribe';
}

function subscribe_admin_subscription_pull($sid) {
  $subscription = subscribe_subscription_load($sid);
  if (!$subscription) {
    drupal_set_message(t('Unable to load the specified subscription.'));
    drupal_goto('admin/content/subscribe');
  }
  
  subscribe_xmlc_pull($subscription->url, $subscription->sid);
  drupal_goto('admin/content/subscribe');
}

function subscribe_admin_form_node_settings($form_state, $node) {
  $exclude = db_result(db_query('SELECT exclude FROM {subscribe_node} WHERE nid = %d', $node->nid));
  if ($exclude === FALSE) {
    $result = db_query('SELECT sid, name FROM {subscribe_subscriptions}');
    while ($row = db_fetch_object($result)) {
      $subscriptions[$row->sid] = $row->name;
    }
    
    $form['remote_info'] = array(
      '#value' => t('If this node has a correspondent on the remote site then you can use this form to manually connect them.'),
      '#prefix' => '<div>',
      '#suffix' => '</div>',
    );

    $form['remote_nid'] = array(
      '#type' => 'textfield',
      '#title' => t('Remote node id'),
      '#description' => t('The nid of the remote node.'),
      '#size' => 40,
      '#maxlength' => 255,
      '#required' => TRUE,
    );

    $form['sid'] = array(
      '#type' => 'select',
      '#title' => t('Subscription'),
      '#description' => t('The subscription which this node belongs to.'),
      '#maxlength' => 255,
      '#options' => $subscriptions,
    );
    
    $form['nid'] = array(
      '#type' => 'hidden',
      '#value' => $node->nid,
    );

    $form['op'] = array(
      '#type' => 'hidden',
      '#value' => 'remote_sync',
    );

    $form['save'] = array(
      '#type' => 'submit',
      '#value' => t('Save settings'),
    );
  }
  else {
    $form['exclude_info'] = array(
      '#value' => t('Nodes that have been created by Pub/Sub and modified locally are automatically excluded from future syncing to prevent overwriting the changes. You can also manually exclude nodes by using this form.'),
      '#prefix' => '<div>',
      '#suffix' => '</div>',
    );

    $form['exclude'] = array(
      '#type' => 'checkbox',
      '#title' => t('Exclude this node from future syncing.'),
      '#default_value' => $exclude,
    );

    $form['nid'] = array(
      '#type' => 'hidden',
      '#value' => $node->nid,
    );

    $form['op'] = array(
      '#type' => 'hidden',
      '#value' => 'exclude',
    );

    $form['save'] = array(
      '#type' => 'submit',
      '#value' => t('Save settings'),
    );
  }
    
  return $form;
}

function subscribe_admin_form_node_settings_submit($form_id, $form_state) {
  if ($form_state['values']['op'] == 'exclude') {
    $nid = $form_state['values']['nid'];
    $exclude = $form_state['values']['exclude'];

    db_query('UPDATE {subscribe_node} SET exclude = %d WHERE nid = %d', $exclude, $nid);
  }
  else {
    $nid = $form_state['values']['nid'];
    $remote_nid = $form_state['values']['remote_nid'];
    $sid = $form_state['values']['sid'];

    db_query('INSERT INTO {subscribe_node} (nid, sid, remote_nid, remote_uid, remote_author, exclude) VALUES (%d, %d, %d, %d, "%s", %d)', $nid, $sid, $remote_nid, 0, "", 0);
  }
  
  drupal_set_message(t('The settings have been saved.'));
}

function subscribe_admin_channel_form_edit($form_state, $sid) {
  $subscription = subscribe_subscription_load($sid);
  if (!$subscription) {
    drupal_set_message(t('Unable to load the specified subscription.'));
    drupal_goto('admin/content/subscribe');
  }
  
  return array('info' => array('#type' => 'markup', '#value' => t('To be added')));
}

function subscribe_admin_settings($form_state) {
  $form = array();
  
  $form['subscribe_upload'] = array(
    '#type' => 'checkbox',
    '#title' => t('Import uploaded files'),
    '#description' => t('Files attached to nodes will be imported into the site. You will need the upload module enabled and the content types set to allow uploads. <strong>Warning:</strong> When a node is updated all its uploads will be deleted and replaced with the new ones.'),
    '#size' => 40,
    '#maxlength' => 255,
    '#default_value' => variable_get('subscribe_upload', 0),
  );
  
  $form['subscribe_enable_deletion'] = array(
    '#type' => 'checkbox',
    '#title' => t('Delete nodes'),
    '#default_value' => variable_get('subscribe_enable_deletion', 0),
    '#description' => t('When a node is deleted on the publisher site delete the node on this site as well.')
  );
  
    
  return system_settings_form($form);
}